from cornice.resource import resource, view
from pyramid.exceptions import NotFound

from unicore.ask.service.models import Question


def get_app_object(request):
    uuid = request.matchdict['uuid']
    question = request.db.query(Question).get(uuid)

    if question is None:
        raise NotFound

    return question


@resource(collection_path='/questions', path='/questions/{uuid}')
class QuestionResource(object):

    def __init__(self, request):
        self.request = request

    @view(renderer='json')
    def collection_post(self):
        return {}

    @view(renderer='json')
    def get(self):
        question = get_app_object(self.request)
        return question.to_dict()

    @view(renderer='json')
    def put(self):
        return {}



import os
from mock import patch
from ConfigParser import ConfigParser
from unittest import TestCase

from sqlalchemy.orm import sessionmaker
from alembic import command as alembic_command
from alembic.config import Config

from webtest import TestApp

from unicore.ask.service import main


config_file_path = os.path.join(os.path.dirname(__file__), 'test.ini')


def get_test_settings():
    here = os.path.dirname(__file__)
    config = ConfigParser()
    config.read(config_file_path)
    settings = dict(config.items('app:unicore.ask.service',
                                 vars={'here': here}))
    return (here, config_file_path, settings)


def get_alembic_config(alembic_dir):
    config = Config(os.path.join(alembic_dir, '../../../../alembic.ini'))
    config.set_main_option('script_location', alembic_dir)
    config.set_main_option('pyramid_config_file', config_file_path)
    return config


def make_app(working_dir, config_file_path, settings,
             extra_environ={}):
    app = TestApp(main({
        '__file__': config_file_path,
        'here': working_dir,
    }, **settings), extra_environ=extra_environ)
    return app


class DBTestCase(TestCase):

    def create_model_object(self, model, session=None, **attrs):
        obj = model()
        for key, val in attrs.iteritems():
            setattr(obj, key, val)

        if session is not None:
            session.add(obj)

        return obj

    @classmethod
    def setUpClass(cls):
        # bind sessions to single database connection
        def sessionmaker_in_transaction(bind):
            cls.connection = bind.connect()
            return sessionmaker(bind=cls.connection)

        cls.transaction_context = patch(
            'unicore.ask.service.sessionmaker',
            new=sessionmaker_in_transaction)
        cls.transaction_context.start()

        # set up app
        working_dir, config_file_path, settings = get_test_settings()
        cls.working_dir = working_dir
        cls.config_file_path = config_file_path
        cls.settings = settings
        cls.app = make_app(
            working_dir=working_dir,
            config_file_path=config_file_path,
            settings=settings)
        cls.sessionmaker = cls.app.app.registry.dbmaker

        # migrate the database
        cls.alembic_config = get_alembic_config(
            alembic_dir=os.path.join(working_dir, '../alembic'))
        alembic_command.upgrade(cls.alembic_config, 'head')

    @classmethod
    def tearDownClass(cls):
        alembic_command.downgrade(cls.alembic_config, 'base')
        cls.connection.close()
        cls.transaction_context.stop()

    def setUp(self):
        self.app.reset()
        self.transaction = self.__class__.connection.begin()
        self.db = self.__class__.sessionmaker()

    def tearDown(self):
        self.db.close()
        self.transaction.rollback()



"""add response count field

Revision ID: 2a4397511296
Revises: 11ec5eb390f4
Create Date: 2015-03-12 17:15:03.910473

"""

# revision identifiers, used by Alembic.
revision = '2a4397511296'
down_revision = '11ec5eb390f4'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('question_options', sa.Column('responses_count', sa.Integer(), nullable=True))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('question_options', 'responses_count')
    ### end Alembic commands ###



"""initial

Revision ID: 11ec5eb390f4
Revises:
Create Date: 2015-03-12 16:08:13.250103

"""

# revision identifiers, used by Alembic.
revision = '11ec5eb390f4'
down_revision = None
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import sqlalchemy_utils


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.create_table('questions',
    sa.Column('uuid', sqlalchemy_utils.types.uuid.UUIDType(binary=False), nullable=False),
    sa.Column('title', sa.Unicode(length=255), nullable=False),
    sa.Column('short_name', sa.Unicode(length=255), nullable=True),
    sa.Column('multiple', sa.Boolean(create_constraint=255), nullable=True),
    sa.Column('question_type', sa.Unicode(length=255), nullable=False),
    sa.PrimaryKeyConstraint('uuid')
    )
    op.create_table('question_options',
    sa.Column('uuid', sqlalchemy_utils.types.uuid.UUIDType(binary=False), nullable=False),
    sa.Column('title', sa.Unicode(length=255), nullable=True),
    sa.Column('short_name', sa.Unicode(length=255), nullable=True),
    sa.Column('question_id', sqlalchemy_utils.types.uuid.UUIDType(binary=False), nullable=False),
    sa.ForeignKeyConstraint(['question_id'], ['questions.uuid'], ),
    sa.PrimaryKeyConstraint('uuid')
    )
    op.create_table('question_responses',
    sa.Column('uuid', sqlalchemy_utils.types.uuid.UUIDType(binary=False), nullable=False),
    sa.Column('text', sa.Unicode(length=255), nullable=False),
    sa.Column('question_option_id', sqlalchemy_utils.types.uuid.UUIDType(binary=False), nullable=False),
    sa.ForeignKeyConstraint(['question_option_id'], ['question_options.uuid'], ),
    sa.PrimaryKeyConstraint('uuid')
    )
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('question_responses')
    op.drop_table('question_options')
    op.drop_table('questions')
    ### end Alembic commands ###



import uuid

from unicore.ask.service.tests import DBTestCase
from unicore.ask.service.models import QuestionOption, Question


class QuestionApiTestCase(DBTestCase):
    def create_question_option(self, session=None, **attrs):
        return self.create_model_object(QuestionOption, session, **attrs)

    def create_question(self, session=None, **attrs):
        return self.create_model_object(Question, session, **attrs)

    def setUp(self):
        super(QuestionApiTestCase, self).setUp()
        self.question_1 = self.create_question(
            self.db, title='What is your name', short_name='name',
            question_type='free_text',
            options=[])
        self.db.flush()
        self.question_1_option = self.create_question_option(
            self.db, question_id=self.question_1.uuid)

        self.question_2 = self.create_question(
            self.db, title='What is your age', short_name='age',
            question_type='multiple_choice',
            options=[])
        self.db.flush()

        self.create_question_option(
            self.db, title='<18', question_id=self.question_2.uuid)
        self.create_question_option(
            self.db, title='18-29', question_id=self.question_2.uuid)
        self.create_question_option(
            self.db, title='30-49', question_id=self.question_2.uuid)
        self.create_question_option(
            self.db, title='50+', question_id=self.question_2.uuid)

        self.question_3 = self.create_question(
            self.db, title='Which sports do you watch', short_name='sports',
            multiple=True, question_type='multiple_choice',
            options=[])
        self.db.flush()

        self.create_question_option(
            self.db, title='cricket', question_id=self.question_3.uuid)
        self.create_question_option(
            self.db, title='rugby', question_id=self.question_3.uuid)
        self.create_question_option(
            self.db, title='soccer', question_id=self.question_3.uuid)
        self.create_question_option(
            self.db, title='tennis', question_id=self.question_3.uuid)
        self.create_question_option(
            self.db, title='other', question_id=self.question_3.uuid)

        self.db.commit()

    def test_uuid(self):
        the_uuid = self.question_1._uuid
        self.assertEqual(the_uuid, uuid.UUID(self.question_1.uuid))
        self.question_1.uuid = self.question_1.uuid
        self.assertEqual(the_uuid, self.question_1._uuid)
        self.question_1.uuid = uuid.UUID(self.question_1.uuid)
        self.assertEqual(the_uuid, self.question_1._uuid)

    def test_question_not_found(self):
        self.app.get('/questions/%s' % uuid.uuid4(), status=404)

    def test_free_text_question(self):
        resp = self.app.get(
            '/questions/%s' % self.question_1.uuid)
        self.assertEqual(resp.status_int, 200)
        self.assertEqual(resp.json_body, self.question_1.to_dict())

    def test_multiple_choice_question(self):
        resp = self.app.get(
            '/questions/%s' % self.question_2.uuid)
        self.assertEqual(resp.status_int, 200)
        self.assertEqual(resp.json_body, self.question_2.to_dict())

    def test_multiple_choice_question_with_multiple_response(self):
        resp = self.app.get(
            '/questions/%s' % self.question_3.uuid)
        self.assertEqual(resp.status_int, 200)
        self.assertEqual(resp.json_body, self.question_3.to_dict())

    def test_edit(self):
        # change non-privileged fields
        resp = self.app.put_json(
            '/questions/%s' % uuid.uuid4().hex,
            params={'title': 'foo2'})
        self.assertEqual(resp.status_int, 200)
        self.assertEqual(resp.json_body, {})

    def test_create(self):
        data = {'title': 'foobar'}
        resp = self.app.post_json(
            '/questions', params=data)
        self.assertEqual(resp.json_body, {})



import logging

from pyramid.config import Configurator
from pyramid_beaker import set_cache_regions_from_settings
from sqlalchemy import engine_from_config
from sqlalchemy.orm import sessionmaker
from sqlalchemy.ext.declarative import declarative_base


logger = logging.getLogger(__name__)

Base = declarative_base()


def db(request):
    maker = request.registry.dbmaker
    session = maker()

    def cleanup(request):
        if request.exception is not None:
            session.rollback()
        else:
            session.commit()
        session.close()

    request.add_finished_callback(cleanup)

    return session


def main(global_config, **settings):
    """ This function returns a Pyramid WSGI application.
    """
    config = Configurator(settings=settings)

    # sqlalchemy setup
    engine = engine_from_config(settings)
    config.registry.dbmaker = sessionmaker(bind=engine)
    # NOTE: db session is tied to request lifespan
    config.add_request_method(db, reify=True)

    # beaker and cache setup
    set_cache_regions_from_settings(settings)

    config.scan()
    return config.make_wsgi_app()



from uuid import uuid4, UUID

from sqlalchemy import Column, Unicode, Boolean, ForeignKey, Integer
from sqlalchemy.orm import relationship
from sqlalchemy_utils import UUIDType


from unicore.ask.service import Base


class UUIDMixin(object):
    _uuid = Column(
        UUIDType(binary=False), name='uuid', default=uuid4,
        primary_key=True)

    @property
    def uuid(self):
        return self._uuid.hex

    @uuid.setter
    def uuid(self, value):
        if isinstance(value, UUID):
            self._uuid = value
        else:
            self._uuid = UUID(hex=value)


class QuestionResponse(Base, UUIDMixin):
    __tablename__ = 'question_responses'

    text = Column(Unicode(255), nullable=False)
    question_option_id = Column(
        UUIDType(binary=False), ForeignKey('question_options.uuid'),
        nullable=False)

    def to_dict(self):
        return {
            'uuid': self.uuid,
            'text': self.text,
        }


class QuestionOption(Base, UUIDMixin):
    __tablename__ = 'question_options'

    title = Column(Unicode(255), nullable=True)
    short_name = Column(Unicode(255), nullable=True)
    responses_count = Column(Integer())
    question_id = Column(
        UUIDType(binary=False), ForeignKey('questions.uuid'), nullable=False)
    responses = relationship(
        QuestionResponse, backref='question_options', lazy="dynamic")

    def to_dict(self):
        return {
            'uuid': self.uuid,
            'title': self.title,
            'short_name': self.short_name,
            'responses_count': self.responses_count,
        }


class Question(Base, UUIDMixin):
    __tablename__ = 'questions'

    question_types = (
        'free_text',
        'multiple_choice',
    )

    title = Column(Unicode(255), nullable=False)
    short_name = Column(Unicode(255), nullable=True)
    multiple = Column(Boolean(255), default=True)
    question_type = Column(Unicode(255), nullable=False)
    options = relationship(QuestionOption, backref='questions', lazy="dynamic")

    def to_dict(self):
        return {
            'uuid': self.uuid,
            'title': self.title,
            'short_name': self.short_name,
            'multiple': self.multiple,
            'question_type': self.question_type,
            'options': [option.to_dict() for option in self.options]
        }
